from Cryptodome.Cipher import AES #line:32
from Cryptodome.PublicKey import RSA
from Cryptodome.Cipher import PKCS1_OAEP
import subprocess,threading,sys,os
from socket import *
from getpass import *
from datetime import datetime
from asyncio import *
import PyQt5
from hashlib import blake2b
from argon2 import PasswordHasher
import msvcrt,re,secrets,base64,requests

__all__=['Longinus']

class Longinus:
    TokenDB:dict=dict()
    #def Token_generator(self,length:int=32):
        #self.length=length
        #try:
        #if (length == 8 or length == 16 or  length== 32):
            #self.hash = blake2b(digest_size=self.length)
            #self.UserID=os.urandom(length)
            #self.Token=base64.b64encode(str({b'userid':self.UserID,b' timestamp':(str(datetime.now().timestamp()).split(".")[0]).encode(),b' external ip':re.search(r'IP Address : (\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})', requests.get('http://ipconfig.kr').text)[1].encode(),b' internal ip':gethostbyname(gethostname()).encode()}).encode())
            #self.size = len(self.Token)
            #self.secrets_token = secrets.token_bytes(16)
            #self.Random_Token = bytearray()
            #for i in range(self.size):
                #self.Random_Token.append(self.Token[i]^self.secrets_token[i%16]),(bytes(a ^ b for a, b in zip( self.Token,self.Userpwrd)
            #self.hash.update(self.Random_Token)
            #self.Random_Token=self.hash.digest()
            #self.Token_secrets_token.setdefault(self.Random_Token,self.secrets_token)
        #else:
            #raise Exception("Token length input error: Token length must be 8 or 16 or 32")
        #except TypeError as e:
           #return e
        #return self.Random_Token
    def Token_generator(self,length:int=28,set_addres:str=None):
            self.length=length
            self.Usera_addres=set_addres
            if self.Usera_addres==None:
                raise Exception("User address cannot be None")
            #elif (length == 8 or length == 16 or  length== 32 and self.Usera_addres!=None):
            else:
                self.UserID=secrets.token_bytes(length);self.hash = blake2b(digest_size=self.length);self.hash.update(self.UserID);self.Token=self.hash.digest();self.Random_Token = bytearray()
                for i in range(self.hash.digest_size):
                    self.Random_Token.append(self.Token[i]^self.UserID[i%self.length])
                self.Token_data={'Time Stamp':(str(datetime.now().timestamp())),'User addres':self.Usera_addres,'login':False}
                self.TokenDB[bytes(self.Random_Token)]=self.Token_data
                return bytes(self.Random_Token),self.Token_data,self.TokenDB

    def Create_RSA_key(self,length:int=2048):  
        self.length=length
        try:
            if (length == 1024 or length == 2048 or  length== 4096 or  length==8192):
                self.key = RSA.generate(length)
                self.private_key = self.key.export_key()
                self.file_out = open("private_key.pem", "wb")
                self.file_out.write(self.private_key)
                self.file_out.close()
                self.public_key = self.key.publickey().export_key()
                self.file_out = open("public_key.pem", "wb")
                self.file_out.write(self.public_key)
                self.file_out.close()
                #self.path=os.path.dirname( os.path.abspath( __file__ ) )
                self.path=os.getcwd()
            else:
                raise Exception("Key length input error: Token length must be 1024 or 2048 or 4096 or 8192")
        except TypeError as e:
            raise Exception(str(e))
        return {"private_key":self.path+"\\"+"private_key.pem","public_key":self.path+"\\"+"public_key.pem"}

    def pwd_hashing(self,pwd):
        ph=PasswordHasher()
        data=pwd
        data=ph.hash(data)
        return data
    
    def token_verifier(self,Token:bytes):
        self.token=Token
        for k,v in self.TokenDB.items():
            if self.token ==k:
                return k,v
            else:
                return False
    
    def login_verifier(self,Token:bytes):
        self.token=Token
        temp,self.val=self.token_verifier(self.token)
        if self.val != False:
            if self.val['login']!=False:
                return True
            else:
                return False

    def file_checker(self,Route:str):
        self.Route=Route
        try:
            with open(self.Route,'r') as f:
                self.filedata=f.readlines()
                for line in self.filedata:
                    if ' | Token | ' in line:
                        return True
                    else:
                        return False
        except:
            return False

    def token_remover(self,Token):
        self.Token=Token
        del self.TokenDB[self.Token]
        return 'Done'

    def DB_saver(self,Token:bytes,Route:str):
        self.token=Token
        self.Route=Route
        key,self.val=self.token_verifier(self.token)
        if  self.file_checker(self.Route) == True:
            with open(self.Route,'a') as f:
                f.write('\n')
                f.write(str(key)+' | Token | '+str(self.val)+' | Token_data | ')
        else:
            with open(self.Route,'w') as f:
                f.write(str(key)+' | Token | '+str(self.val)+' | Token_data | ')
        
    def DB_loader(self,Route:str):
        self.Route=Route
        if  self.file_checker(self.Route) == True:
            with open(self.Route,'r') as f:
                self.rdata=f.readlines()
            return self.rdata

    def Token_loader(self,read_data):
        self.rdata=read_data
        for line in self.rdata:
            for num in range(len(line.split(' | '))):
                a=eval(line.split(' | ')[0]);b=eval(line.split(' | ')[2])
                self.TokenDB.setdefault(a,b)
        return self.TokenDB


    class authority_editor:
        def constructor(self,Token:bytes,rank:int):
            self.token=Token;self.rank=rank



    def bypass():
        pass


from LonginusP import *
from Cryptodome.Cipher import AES #line:32
from Cryptodome.PublicKey import RSA
from Cryptodome.Cipher import AES, PKCS1_OAEP
import subprocess,threading,sys,os
from socket import *
from getpass import *
from datetime import datetime
from asyncio import *
import PyQt5
from hashlib import blake2b
from argon2 import PasswordHasher
import msvcrt,re,secrets,secrets,base64,requests,struct

__all__=['Server']

class Server:

    L= Longinus()
    Login_list:list=list()
    def __init__(self,set_addr:str="0.0.0.0",set_port:int=9997):
        self.s=socket()
        self.s.bind((set_addr,set_port))
        self.s.listen(0)
        while True:
            #try:
            self.address=list()
            self.head,self.c,self.addr=self.recv_head()
            self.address.append(self.addr)
            self.pul_key=self.recv_keys()
            self.Token,self.Token_data,self.Token_DB=self.L.Token_generator(length=32,set_addres=self.addr)
            self.Token_RSA=self.Encryption_token(self.Token,self.pul_key)
            self.send_server()
            print(self.Token)
            print(self.Token_data)
            self.head=self.c.recv(4);self.head=int(str(struct.unpack("I",self.head)).split(',')[0].split('(')[1])
            self.userdata=self.recv_server()
            self.userdata=self.Decryption(self.userdata)
            self.userdata=self.user_data_decompress(self.Token,self.userdata)
            self.userdata=self.SignUp(self.userdata[0],self.userdata[1])
            self.Server_DB:dict=dict()
            self.Server_DB[self.Token]=self.userdata
            print(self.Server_DB)
            self.saveing_all_data()
            #except:
                #print("unexpected error")
                #continue
    def Index():
        pass

    def send_server(self):
        self.c.sendall(self.merge_data())
        return 'done'

    def user_data_decompress(self,token,data):
        self.temp=data;self.Token=token
        self.temp=self.temp[self.Token]
        self.temp=eval(self.temp.decode())
        self.uid=self.temp[0];self.uwp=self.temp[1];self.uwp=self.uwp['userpw']
        self.temp0=bytearray()
        for i in range(len(self.uwp)):
            self.temp0.append(self.uwp[i]^self.Token[i%len(self.Token)])
        self.temp=self.temp[1];self.temp['userpw']=bytes(self.temp0)
        print([self.uid,self.temp])
        return [self.uid,self.temp]
        

    def recv_head(self):
        #try:
        while True:
            print('waiting for client...')
            self.c,self.addr=self.s.accept();
            self.head=self.c.recv(4);self.head=int(str(struct.unpack("I",self.head)).split(',')[0].split('(')[1])
            print('[Received from] : ',str(self.addr))
            return self.head,self.c,self.addr
        #except:
            #print('An unexpected error occurred')

    def merge_data(self):
        self.head=struct.pack("I",len(self.Token_RSA))
        self.send_data=self.head+self.Token_RSA
        print('send : ','\n')
        return self.send_data

    def recv_keys(self):
        while True:
            self.recv_datas=bytes()
            if self.head<2048:
                self.recv_datas=self.c.recv(self.head)
                self.recv_datas=base64.b64decode(self.recv_datas)
                return self.recv_datas
            elif self.head>=2048:
                self.recv_datas=bytearray()
                for i in range(int(self.head/2048)):
                    self.recv_datas.append(self.c.recv(2048))
                    print("Downloading "+str(self.addr)+" : "+str(2048*i/self.head*100)+" %"+" Done...")
                print("Downloading "+str(self.addr)+" Data... : "+"100 % Done...")
                self.recv_datas=base64.b64decode(bytes(self.recv_datas))
                return self.recv_datas

    def recv_server(self):
        while True:
            self.recv_datas=bytes()
            if self.head<2048:
                self.recv_datas=self.c.recv(self.head)
                self.recv_datas=self.recv_datas
                return self.recv_datas
            elif self.head>=2048:
                self.recv_datas=bytearray()
                for i in range(int(self.head/2048)):
                    self.recv_datas.append(self.c.recv(2048))
                    print("Downloading "+str(self.addr)+" : "+str(2048*i/self.head*100)+" %"+" Done...")
                print("Downloading "+str(self.addr)+" Data... : "+"100 % Done...")
                self.recv_datas=bytes(self.recv_datas)
                return self.recv_datas     
        #except:
            #print('An unexpected error occurred')

    def Decryption(self,data:bytes):
        self.data=data
        self.temp=list
        public_key = RSA.import_key(open(self.pul_key))
        n=public_key.size_in_bytes()
        nonce=self.data[n:n+16]
        tag=self.data[n+16:n+32]
        ciphertext =self.data[n+32:-1]+self.data[len(self.data)-1:]
        #print(enc_session_key,'\n',nonce,'\n',tag,'\n',ciphertext)
        #print(print(ciphertext+self.data[len(self.data)-1:]))
        # Decrypt the session key with the private RSA key
        session_key = self.Token
        # Decrypt the data with the AES session key
        cipher_aes = AES.new(session_key, AES.MODE_EAX, nonce)
        data = cipher_aes.decrypt_and_verify(ciphertext, tag)
        return data


    def SignUp(self,UserID:dict,User_pwrd:dict):
        self.UserID=UserID['userid']
        self.Userpwrd=User_pwrd['userpw']
        if (" " not in self.UserID and "\r\n" not in self.UserID and "\n" not in self.UserID and "\t" not in self.UserID and re.search('[`~!@#$%^&*(),<.>/?]+', self.UserID) is None):
            if len( self.Userpwrd.decode()) > 8 and re.search('[0-9]+', self.Userpwrd.decode()) is not None and re.search('[a-zA-Z]+', self.Userpwrd.decode()) is not None and re.search('[`~!@#$%^&*(),<.>/?]+', self.Userpwrd.decode()) is not None and " " not in self.Userpwrd.decode() :
                self.Userpwrd=self.L.pwd_hashing(base64.b64encode(bytes(a ^ b for a, b in zip( self.Token,self.Userpwrd))))
                self.login_data=[{'userid':self.UserID},{'userpw':self.Userpwrd}]
                return [{'userid':self.UserID},{'userpw':self.Userpwrd}]
            else:
                print(User_pwrd.decode())
                raise  Exception("Your password is too short or too easy. Password must be at least 8 characters and contain numbers, English characters and symbols. Also cannot contain whitespace characters.")
        else:
            raise  Exception("Name cannot contain spaces or special characters")

    def ReSign(self,Token:bytes):
        pass
#############################################################################################################################################################################################################################
    def Login(self,UserName:str,User_pwrd:bytes):
        self.hash = blake2b(digest_size=32)
        self.UserName=UserName
        self.Userpwrd=User_pwrd
        if (" " not in self.UserName and "\r\n" not in self.UserName and "\n" not in self.UserName and "\t" not in self.UserName and re.search('[`~!@#$%^&*(),<.>/?]+', self.UserName) is None):
            if len( self.Userpwrd.decode()) > 8 and re.search('[0-9]+', self.Userpwrd.decode()) is not None and re.search('[a-zA-Z]+', self.Userpwrd.decode()) is not None and re.search('[`~!@#$%^&*(),<.>/?]+', self.Userpwrd.decode()) is not None and " " not in self.Userpwrd.decode() :
                self.hash.update(base64.b64encode(bytes(a ^ b for a, b in zip( self.Userpwrd, self.Token))))
                self.Userpwrd=PasswordHasher().hash(self.hash.digest())
                self.Server_DB.setdefault(self.Token,{self.UserName:self.Userpwrd})
                return {self.Token:{self.UserName:self.Userpwrd}}
            else:
                print(User_pwrd.decode())
                raise  Exception("Your password is too short or too easy. Password must be at least 8 characters and contain numbers, English characters and symbols. Also cannot contain whitespace characters.")
        else:
            raise  Exception("Name cannot contain spaces or special characters")
#############################################################################################################################################################################################################################
    def Logout():
        pass
    def Rename():
        pass
    def Repwrd():
        pass
    def verify():
        pass

    def token_remover(self,Token):
        self.Token=Token
        del self.Token_DB[self.Token]
        return 'done'

    def Encryption_token(self,Token:bytes,set_public_key:bytes):
        self.Token =Token
        public_key = RSA.import_key(set_public_key)
        cipher_rsa = PKCS1_OAEP.new(public_key)
        self.Token_RSA = cipher_rsa.encrypt(base64.b64encode(self.Token))
        return self.Token_RSA

    def Decryption(self,data:bytes):
        self.data=data
        #print(self.data)
        self.temp=list
        nonce=self.data[:16]
        tag=self.data[16:32]
        ciphertext =self.data[32:-1]+self.data[len(self.data)-1:]
        #print(nonce,'\n',tag,'\n',ciphertext)
        session_key = self.Token
        cipher_aes = AES.new(session_key, AES.MODE_EAX, nonce)
        data = base64.b64decode(cipher_aes.decrypt_and_verify(ciphertext, tag))
        return {session_key:data}

    def saveing_all_data(self,Route=r'C:\Users\Eternal_Nightmare0\Desktop\Project-Longinus\package\LonginusPYPL\ServerDB.DB'):
        self.L.DB_saver(self.Token,Route)
        if  self.L.file_checker(Route) == True:
            with open(Route,'a') as f:
                f.write('\n')
                f.write(str(self.Token)+' | Token | '+str(self.userdata)+' | User_data | ')
        else:
            with open(Route,'w') as f:
                f.write(str(self.Token)+' | Token | '+str(self.userdata)+' | User_data | ')
Server()


import setuptools

with open(r"README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="LonginusPYPl",
    version="0.1.8",
    author="Example Author",
    author_email="team.longinus.project@gmail.com",
    description="A small example package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/projectlonginus/LonginusPYPI",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
    python_requires='>=3.11',
)


from LonginusP import *
from Cryptodome.Cipher import AES #line:32
from Cryptodome.PublicKey import RSA
from Cryptodome.Cipher import AES, PKCS1_OAEP
import subprocess,threading,sys,os
from socket import *
from getpass import *
from datetime import datetime
from asyncio import *
import PyQt5
from hashlib import blake2b
from argon2 import PasswordHasher
import msvcrt,re,secrets,secrets,base64,requests
import json
import struct

__all__=['Client']

class Client:
    L=Longinus()
    ClientDB:dict=dict()
    def __init__(self,set_address:str='127.0.0.1',set_port:int=9997):
        self.address=set_address;self.port=set_port;
        self.s=socket()
        self.s.connect((self.address,self.port))
        self.set_keys:dict=self.setting_keys()
        self.send_keys()
        self.Token=self.recv_client()
        self.uid,self.upw=self.user_injecter()
        self.udata=self.SignUp(self.Token,self.uid,self.upw)
        self.udata=self.Encryption(self.udata)
        self.send_userdata(self.udata)
    def Index(self,Token:bytes):
        pass

    def send_keys(self):
        with open(self.set_keys['public_key'],'r') as kfc:
            self.body=base64.b64encode(kfc.read().encode())
        self.s.sendall(self.merge_data(self.body))

    def merge_data(self,data):
        self.body=data
        self.head=struct.pack("I",len(self.body))
        self.send_data=self.head+self.body
        return self.send_data

    def send_userdata(self,data):
        self.udata=data
        self.s.sendall(self.merge_data(self.udata))
        
        

    def recv_client(self):
        while True:
            self.head=self.s.recv(4);self.head=int(str(struct.unpack("I",self.head)).split(',')[0].split('(')[1])
            self.recv_datas=bytearray()
            if self.head==self.head:
                self.recv_datas=self.Decryption_Token(self.s.recv(self.head))
                print('Token Issued : ',self.recv_datas)
                print(len(self.recv_datas))
                #self.send_client(self.recv_datas)
                return self.recv_datas
            else:
                for i in range(int(self.head/2048)):
                    self.recv_datas.append(self.Decryption_Token(self.s.recv(2048)))
                    print("Downloading "+str(self.addr)+" : "+str(2048*i/self.head*100)+" %"+" Done...")
                print("Downloading "+str(self.addr)+" Data... : "+"100 % Done...")
                print('downloaded data : ',self.recv_datas)
                return self.recv_datas


    def SignUp(self,Token:bytes,UserID:str,User_pwrd:bytes):
        self.Token=Token
        self.UserID=UserID
        self.Userpwrd=User_pwrd
        self.temp_data=bytearray()
        if (" " not in self.UserID and "\r\n" not in self.UserID and "\n" not in self.UserID and "\t" not in self.UserID and re.search('[`~!@#$%^&*(),<.>/?]+', self.UserID) is None):
            if self.user_checker(UserID)==False:
                if len( self.Userpwrd.decode()) > 8 and re.search('[0-9]+', self.Userpwrd.decode()) is not None and re.search('[a-zA-Z]+', self.Userpwrd.decode()) is not None and re.search('[`~!@#$%^&*(),<.>/?]+', self.Userpwrd.decode()) is not None and " " not in self.Userpwrd.decode() :
                    for i in range(len(self.Userpwrd)):
                        self.temp_data.append(self.Userpwrd[i]^self.Token[i%len(self.Token)])
                    self.login_data=[{'userid':self.UserID},{'userpw':bytes(self.temp_data)}]
                    return self.login_data
                else:
                    print(User_pwrd.decode())
                    raise  Exception("Your password is too short or too easy. Password must be at least 8 characters and contain numbers, English characters and symbols. Also cannot contain whitespace characters.")
            else:
                raise  Exception("A user with the same name already exists. Please change the name.")
        else:
            raise  Exception("Name cannot contain spaces or special characters")


    def setting_keys(self,len:int=2048):
        self.set_keys=self.L.Create_RSA_key()
        return self.set_keys


    def Encryption(self,Token:bytes,data:bytes):
        self.Token =Token
        self.data=str(data).encode()
        self.send_data=bytes
        session_key = self.Token
        # Encrypt the data with the AES session key
        cipher_aes = AES.new(session_key, AES.MODE_EAX)
        ciphertext, tag = cipher_aes.encrypt_and_digest(base64.b64encode(self.data))
        self.send_data= cipher_aes.nonce+ tag+ ciphertext
        return self.send_data


    def ReSign(self,Token:bytes):
        pass

    def Login():
        pass

    def Logout():
        pass

    def user_checker(self,UserID:str):
        self.UserID=UserID
        self.Userdata=self.ClientDB.values()
        if self.UserID in self.Userdata:
            return True
        else:
            return False

    def Rename():
        pass

    def Repwrd():
        pass

    def token_verifier():
        pass

    def verify():
        pass

    def emall_verify():
        pass

    
    def check_DB(self):
        if self.ClientDB==None:
            self.ClientDB.setdefault(self.user_injecter())
            return self.ClientDB
        else:
            return self.ClientDB

    def Encryption(self,data:bytes):
        self.data=str(data).encode()
        self.send_data=bytes
        session_key = self.Token
        cipher_aes = AES.new(session_key, AES.MODE_EAX)
        ciphertext, tag = cipher_aes.encrypt_and_digest(base64.b64encode(self.data))
        self.send_data= cipher_aes.nonce+ tag+ ciphertext
        print(cipher_aes.nonce,'\n',tag,'\n',ciphertext)
        return self.send_data

    def Decryption_Token(self,Token):
        self.Token=Token
        private_key = RSA.import_key(open(self.set_keys['private_key']).read())
        cipher_rsa = PKCS1_OAEP.new(private_key)
        session_key = base64.b64decode(cipher_rsa.decrypt(self.Token))
        return session_key

    def user_injecter(self):
        self.pwrd=bytes()
        self.userid=input("Please enter your name to sign up : ")
        self.input_num=0
        print("Please enter your password to sign up : ",end="",flush=True)
        while True:
            self.new_char=msvcrt.getch()
            if self.new_char==b'\r':
                break
            elif self.new_char==b'\b':
                if self.input_num < 1:
                    pass
                else:
                    msvcrt.putch(b'\b')
                    msvcrt.putch(b' ')
                    msvcrt.putch(b'\b')
                    self.pwrd+=self.new_char
                    self.input_num-=1
            else:
                print("*",end="", flush=True)
                self.pwrd+=self.new_char
                self.input_num+=1
        return self.userid,self.pwrd
    
Client()